package com.ruoyi.system.mapper;

import java.util.List;

import com.ruoyi.system.domain.TEarlyWarning;

/**
 * 【请填写功能名称】Mapper接口
 *
 * @author ruoyi
 * @date 2023-09-27
 */
public interface TEarlyWarningMapper {
    /**
     * 查询【请填写功能名称】
     *
     * @param id 【请填写功能名称】主键
     * @return 【请填写功能名称】
     */
    public TEarlyWarning selectTEarlyWarningById(Long id);

    /**
     * 查询【请填写功能名称】
     *
     * @param alarmId 【请填写功能名称】主键
     * @return 【请填写功能名称】
     */
    public List<TEarlyWarning> selectTEarlyWarningByAlarmId(Long alarmId);

    /**
     * 查询【请填写功能名称】列表
     *
     * @param tEarlyWarning 【请填写功能名称】
     * @return 【请填写功能名称】集合
     */
    public List<TEarlyWarning> selectTEarlyWarningList(TEarlyWarning tEarlyWarning);

    /**
     * 新增【请填写功能名称】
     *
     * @param tEarlyWarning 【请填写功能名称】
     * @return 结果
     */
    public int insertTEarlyWarning(TEarlyWarning tEarlyWarning);

    /**
     * 修改【请填写功能名称】
     *
     * @param tEarlyWarning 【请填写功能名称】
     * @return 结果
     */
    public int updateTEarlyWarning(TEarlyWarning tEarlyWarning);

    /**
     * 删除【请填写功能名称】
     *
     * @param id 【请填写功能名称】主键
     * @return 结果
     */
    public int deleteTEarlyWarningById(Long id);

    /**
     * 批量删除【请填写功能名称】
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteTEarlyWarningByIds(Long[] ids);

    List<TEarlyWarning> selectTEarlyWarningList_SOS(TEarlyWarning earlyWarning);

    List<TEarlyWarning> selectTEarlyWarningList_HEALTHSOLVE(TEarlyWarning earlyWarning);

    List<TEarlyWarning> selectTEarlyWarningList_OTHERSOLVE(TEarlyWarning earlyWarning);

    /**
     * 查询类型已完成数量
     *
     * @param type
     * @return
     */
    Integer selectTEarlyWarningCompletedCountByType(Integer type);

    /**
     * 查询类型未完成数量
     *
     * @param type
     * @return
     */
    Integer selectTEarlyWarningUnCompletedCountByType(Integer type);

    /**
     * 查询类型未完成数量
     *
     * @return
     */
    Integer selectTEarlyWarningUnCompletedAllCount();

    /**
     * 查询类型未完成百分比
     *
     * @param type
     * @return
     */
    Double selectTEarlyWarningUnCompletedPercentageByType(Integer type);

    /**
     * 查询类型未完成百分比
     *
     * @return
     */
    Double selectTEarlyWarningUnCompletedPercentageByAllType();
}
